"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const logger_1 = require("../../utils/logger");
const read_files_1 = require("../../utils/file/read-files");
const write_file_1 = require("../../utils/file/write-file");
const splashscreen_1 = require("../../utils/splashscreen");
const setup_fcl_1 = require("../../utils/catalog/setup-fcl");
const generate_interaction_template_1 = require("../../utils/catalog/generate-interaction-template");
const catalog_helpers_1 = require("../../utils/catalog/catalog-helpers");
const generate_save_path_1 = require("../../utils/catalog/generate-save-path");
class Catalog extends core_1.Command {
    async run() {
        const { flags, args } = await this.parse(Catalog);
        const { projects: _projects, path, skip_projects: _skip_projects } = args;
        const projects = _projects ? _projects.split(",") : [];
        const skip_projects = _skip_projects ? _skip_projects.split(",") : [];
        (0, splashscreen_1.catalogSplashTitle)();
        logger_1.logger.warn("⚠️  Warning: The FLIX CLI is currently in early alpha. Please report any bugs by opening an issue here: https://github.com/onflow/flow-interaction-template-tools/issues/new");
        const flowJSONFiles = await (0, read_files_1.readFiles)(flags.flowJsonPath || "flow.json");
        const flowJSON = flowJSONFiles[0] ?
            JSON.parse(flowJSONFiles[0].content) :
            null;
        if (flowJSON === null) {
            logger_1.logger.error("❌ Error: No flow.json file found.");
            return;
        }
        const GENERATED = {};
        const networks = ["mainnet", "testnet"]; // Networks
        let identifiers = projects; // Project identifiers
        let transactions = ["CollectionInitialization", "StorefrontListItem", "StorefrontBuyItem", "StorefrontRemoveItem"]; // Transaction types
        if (identifiers.length === 0) {
            for (const network of networks) {
                if (network === "mainnet") {
                    (0, setup_fcl_1.setupMainnet)();
                }
                if (network === "testnet") {
                    (0, setup_fcl_1.setupTestnet)();
                }
                const _identifiers = await (0, catalog_helpers_1.getAllCollections)();
                identifiers = identifiers.concat(Object.keys(_identifiers));
            }
        }
        try {
            for (const identifier of identifiers) {
                for (const transaction of transactions) {
                    logger_1.logger.default(`\n⚙️  Generating template for ${identifier} ${transaction}`);
                    const existing_templates = await (0, read_files_1.readFiles)(`${(0, generate_save_path_1.generateSavePath)(path)}${identifier}/${identifier}-${transaction}*.template.json`);
                    if (existing_templates.length > 0) {
                        logger_1.logger.default(`\n⚙️  Skipping ${identifier} ${transaction}, already exists`);
                        continue;
                    }
                    if (skip_projects.includes(identifier))
                        continue;
                    const interaction_template = await new Promise(async (res, rej) => {
                        const timeoutId = setTimeout(() => {
                            logger_1.logger.default(`\n⚙️  Aborting ${identifier} ${transaction}, exceeded time`);
                            res(null);
                        }, 120000);
                        const interaction_template = await (0, generate_interaction_template_1.generateInteractionTemplate)(transaction, identifier, networks);
                        clearTimeout(timeoutId);
                        res(interaction_template);
                    });
                    if (!interaction_template)
                        continue;
                    logger_1.logger.default(`\n⚙️ Generated template!`);
                    if (!(interaction_template === null || interaction_template === void 0 ? void 0 : interaction_template.f_type)) {
                        GENERATED[identifier] = {
                            ...GENERATED === null || GENERATED === void 0 ? void 0 : GENERATED[identifier],
                            [transaction]: {
                                ["mainnet"]: interaction_template.mainnet,
                                ["testnet"]: interaction_template.testnet,
                            }
                        };
                        if (interaction_template.mainnet) {
                            await (0, write_file_1.writeFile)(`${(0, generate_save_path_1.generateSavePath)(path)}${identifier}/${identifier}-${transaction}-${"mainnet"}.template.json`, JSON.stringify(interaction_template.mainnet, null, 2));
                        }
                        if (interaction_template.testnet) {
                            await (0, write_file_1.writeFile)(`${(0, generate_save_path_1.generateSavePath)(path)}${identifier}/${identifier}-${transaction}-${"testnet"}.template.json`, JSON.stringify(interaction_template.testnet, null, 2));
                        }
                    }
                    else {
                        GENERATED[identifier] = {
                            ...GENERATED === null || GENERATED === void 0 ? void 0 : GENERATED[identifier],
                            [transaction]: {
                                interaction_template
                            }
                        };
                        await (0, write_file_1.writeFile)(`${(0, generate_save_path_1.generateSavePath)(path)}${identifier}/${identifier}-${transaction}.template.json`, JSON.stringify(interaction_template, null, 2));
                    }
                    logger_1.logger.default(`\n⚙️  Saved template!`);
                }
            }
            await (0, write_file_1.writeFile)(`${(0, generate_save_path_1.generateSavePath)(path)}catalog-manifest.json`, JSON.stringify(GENERATED, null, 2));
            logger_1.logger.default(`\n🌱 Saved templates to ${`${(0, generate_save_path_1.generateSavePath)(path)}catalog-manifest.json`}\n`);
        }
        catch (e) {
            logger_1.logger.error(`\n❌ An error occurred.\n`);
        }
    }
}
exports.default = Catalog;
Catalog.description = "Generate Interaction Templates from NFT Catalog.";
Catalog.examples = ["$ flowplate catalog"];
Catalog.flags = {
    flowJsonPath: core_1.Flags.string({
        char: "f",
        summary: "Path to a flow.json configuration file.",
    }),
};
Catalog.args = [
    {
        name: "path",
        description: "Path to a folder to store the generated Interaction Templates.",
        required: true,
    },
    {
        name: "projects",
        description: "List of NFT Catalog projects to generate Interaction Templates for (default all)",
        required: false,
    },
    {
        name: "skip_projects",
        description: "List of NFT Catalog projects to skip when generating Interaction Templates (default none)",
        required: false,
    }
];
